//============================================================================
//
//   SSSS    tt          lll  lll
//  SS  SS   tt           ll   ll
//  SS     tttttt  eeee   ll   ll   aaaa
//   SSSS    tt   ee  ee  ll   ll      aa
//      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
//  SS  SS   tt   ee      ll   ll  aa  aa
//   SSSS     ttt  eeeee llll llll  aaaaa
//
// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
// and the Stella Team
//
// See the file "License.txt" for information on usage and redistribution of
// this file, and for a DISCLAIMER OF ALL WARRANTIES.
//============================================================================

#ifndef TOOL_TIP_HXX
#define TOOL_TIP_HXX

/**
 * Class for providing tooltip functionality
 *
 * @author Thomas Jentzsch
 */

class OSystem;
class Dialog;
class Widget;
class FBSurface;

#include "Rect.hxx"

class ToolTip
{
  public:
    // Maximum tooltip length
    static constexpr uInt32 MAX_LEN = 80;

    ToolTip(Dialog& dialog, const GUI::Font& font);
    ~ToolTip() = default;

    /**
      Request a tooltip display.
    */
    void request();

    /**
      Hide a displayed tooltip and reset the timer.
    */
    void hide();

    /**
      Hide a displayed tooltip and reset the timer slowly.
      This allows faster tip display of the next tip.
    */
    void release();

    /**
      Update focussed widget and current mouse position.
    */
    void update(const Widget* widget, Common::Point pos);

    /*
      Render the tooltip
    */
    void render();

  private:
    void show();

  private:
    static constexpr uInt32 DELAY_TIME = 45; // display delay

    Dialog& myDialog;
    const GUI::Font& myFont;
    const Widget* myTipWidget{nullptr};
    const Widget* myFocusWidget{nullptr};

    uInt32 myTimer{0};
    Common::Point myPos;
    uInt32 myWidth{0};
    uInt32 myHeight{0};
    uInt32 myTextXOfs{0};
    uInt32 myTextYOfs{0};
    bool myTipShown{false};
    uInt32 myScale{1};
    shared_ptr<FBSurface> mySurface;
};

#endif
